from __future__ import annotations

from .ara.HotelReviewSentimentClassification import *
from .ara.RestaurantReviewSentimentClassification import *
from .ara.TweetEmotionClassification import *
from .ara.TweetSarcasmClassification import *
from .ben.BengaliHateSpeechClassification import *
from .bul.BulgarianSentimentClassification import *
from .bul.BulgarianStoreReviewSentimentClassfication import *
from .ces.CzechSubjectivityClassification import *
from .dan.AngryTweetsClassification import *
from .dan.DalajClassification import *
from .dan.DanishPoliticalCommentsClassification import *
from .dan.DKHateClassification import *
from .dan.LccSentimentClassification import *
from .eng.AmazonPolarityClassification import *
from .eng.Banking77Classification import *
from .eng.EmotionClassification import *
from .eng.ImdbClassification import *
from .eng.NewsClassification import *
from .eng.ToxicConversationsClassification import *
from .eng.TweetSentimentExtractionClassification import *
from .est.estonian_valence import *
from .fas.PersianFoodSentimentClassification import *
from .fra.MovieReviewSentimentClassification import *
from .hin.HindiDiscourseClassification import *
from .hrv.CroatianSentimentClassification import *
from .ind.IndonesianIdClickbaitClassification import *
from .ita.ItaHateClassification import *
from .ita.ItalianLinguistAcceptabilityClassification import *
from .jpn.WRIMEClassification import *
from .kur.KurdishSentimentClassification import *
from .mkd.MacedonianTweetSentimentClassification import *
from .mlt.MalteseSentimentClassification import *
from .multilingual.AmazonCounterfactualClassification import *
from .multilingual.AmazonReviewsClassification import *
from .multilingual.MasakhaNEWSClassification import *
from .multilingual.MassiveIntentClassification import *
from .multilingual.MassiveScenarioClassification import *
from .multilingual.MTOPDomainClassification import *
from .multilingual.MTOPIntentClassification import *
from .multilingual.NordicLangClassification import *
from .multilingual.ScalaClassification import *
from .nld.DutchBookReviewSentimentClassification import *
from .nob.NoRecClassification import *
from .nob.NorwegianParliamentClassification import *
from .pol.PolishClassification import *
from .ron.RomanianSentimentClassification import *
from .slk.SlovakSentimentClassification import *
from .swe.SweRecClassification import *
from .tur.TurkishMovieSentimentClassification import *
from .tur.TurkishProductSentimentClassification import *
from .uig.UyghurSentimentClassification import *
from .vie.VieStudentFeedbackClassification import *
from .zho.CMTEBClassification import *
from .zho.YueOpenriceReviewClassification import (
    YueOpenriceReviewClassification,  # noqa: F401
)
