"""The `LANG_MAPPING` dictionary is mapping bigram, trigram language codes to each others, to include various datasets that use one or the other"""

from __future__ import annotations

LANG_MAPPING = {
    "fr": ["fra", "fra_Latn"],
    "en": ["eng", "eng_Latn"],
    "afr": ["afr_Latn"],
    "amh": ["amh_Ethi"],
    "ara": ["arb_Arab"],
    "asm": ["asm_Beng"],
    "ast": ["ast_Latn"],
    "azj": ["azj_Latn"],
    "bel": ["bel_Cyrl"],
    "ben": ["ben_Beng"],
    "bos": ["bos_Latn"],
    "bul": ["bul_Cyrl"],
    "cat": ["cat_Latn"],
    "ceb": ["ceb_Latn"],
    "ces": ["ces_Latn"],
    "ckb": ["ckb_Arab"],
    "cym": ["cym_Latn"],
    "dan": ["dan_Latn"],
    "deu": ["deu_Latn"],
    "ell": ["ell_Grek"],
    "eng": ["eng_Latn"],
    "est": ["est_Latn"],
    "fin": ["fin_Latn"],
    "fra": ["fra_Latn"],
    "ful": ["fuv_Latn"],
    "gle": ["gle_Latn"],
    "glg": ["glg_Latn"],
    "guj": ["guj_Gujr"],
    "hau": ["hau_Latn"],
    "heb": ["heb_Hebr"],
    "hin": ["hin_Deva"],
    "hrv": ["hrv_Latn"],
    "hun": ["hun_Latn"],
    "hye": ["hye_Armn"],
    "ibo": ["ibo_Latn"],
    "ind": ["ind_Latn"],
    "isl": ["isl_Latn"],
    "ita": ["ita_Latn"],
    "jav": ["jav_Latn"],
    "jpn": ["jpn_Jpan"],
    "kam": ["kam_Latn"],
    "kan": ["kan_Knda"],
    "kat": ["kat_Geor"],
    "kaz": ["kaz_Cyrl"],
    "khm": ["khm_Khmr"],
    "kir": ["kir_Cyrl"],
    "kor": ["kor_Hang"],
    "lao": ["lao_Laoo"],
    "Latvian": ["lij_Latn"],
    "kea": ["lim_Latn"],
    "lin": ["lin_Latn"],
    "lit": ["lit_Latn"],
    "ltz": ["ltz_Latn"],
    "lug": ["lug_Latn"],
    "luo": ["luo_Latn"],
    "lav": ["lvs_Latn"],
    "mal": ["mal_Mlym"],
    "mar": ["mar_Deva"],
    "mkd": ["mkd_Cyrl"],
    "mlt": ["mlt_Latn"],
    "mon": ["khk_Cyrl"],
    "mri": ["mri_Latn"],
    "mya": ["mya_Mymr"],
    "nld": ["nld_Latn"],
    "nob": ["nob_Latn"],
    "npi": ["npi_Deva"],
    "nso": ["nso_Latn"],
    "nya": ["nya_Latn"],
    "oci": ["oci_Latn"],
    "orm": ["gaz_Latn"],
    "ory": ["ory_Orya"],
    "pan": ["pan_Guru"],
    "fas": ["pes_Arab"],
    "pol": ["pol_Latn"],
    "por": ["por_Latn"],
    "pus": ["pbt_Arab"],
    "ron": ["ron_Latn"],
    "rus": ["rus_Cyrl"],
    "slk": ["slk_Latn"],
    "sna": ["sna_Latn"],
    "snd": ["snd_Arab"],
    "som": ["som_Latn"],
    "spa": ["spa_Latn"],
    "srp": ["srp_Cyrl"],
    "swe": ["swe_Latn"],
    "swh": ["swh_Latn"],
    "tam": ["tam_Taml"],
    "tel": ["tel_Telu"],
    "tgk": ["tgk_Cyrl"],
    "tgl": ["tgl_Latn"],
    "tha": ["tha_Thai"],
    "tur": ["tur_Latn"],
    "ukr": ["ukr_Cyrl"],
    "umb": ["umb_Latn"],
    "urd": ["urd_Arab"],
    "uzb": ["uzn_Latn"],
    "vie": ["vie_Latn"],
    "wol": ["wol_Latn"],
    "xho": ["xho_Latn"],
    "yor": ["yor_Latn"],
    "zho_simpl": ["zho_Hans"],
    "zho_trad": ["zho_Hant"],
    "msa": ["zsm_Latn"],
    "zul": ["zul_Latn"],
}
